<?php
/*
 * User:
 * index.php        => getHTML
 * registration.php => getHTMLRegistration
 * 
 * Admin:
 * AddRegistrationUI.inc.php  => booking->getHTMLView(false, 1)
 *                            => user->getHTMLAddReg
 * ViewDataUI.inc.php         => getHTMLView
 */
class RegistrationForm {
  private $title;
  private $header;
  private $error;
  
  private $includeUser;
  private $includeBooking;
  private $sendEmailGlobal;
  private $sendEmailEvent;
  
  private $userData;
  private $uis = array();
  
  private $confirmationText;
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  public function __construct(DbConnection $DB) {
    //Configuration
    $this->includeUser = 0;
    $this->includeBooking = 0;
    $this->sendEmailGlobal = 0;
    $this->sendEmailEvent = 0;
    if(!isset($_SESSION['user']) && !isset($_SESSION['admin'])) {
      $this->includeUser = 1;
      $this->sendEmailGlobal = (KDbConfig::getAllowUserLogin() ? 1 : 0);
    }
    if(isset($_SESSION['event'])) {
      $this->includeBooking = 1;
      $this->sendEmailEvent = 1;
    }
    
    if(!$this->includeUser && !$this->includeBooking) {
      $this->error = _("No event selected.");
      return;
    }
  
    //Setup UIs
    $firstUI = 1;
    if ($this->includeUser) {
      $firstUI = 0;
      $this->uis[0] = new UserUI($DB);
      $this->userData = &$this->uis[0]->data;
    }
    else
      $this->userData = &$_SESSION['user'];
    
    if ($this->includeBooking) {
      $this->uis[1] = new UserBookingUI($DB);
      if($firstUI==0) {
        $this->uis[0]->addValidationRules(UserBookingUI::getValidationRules());
      }
    }
    
    // Include Plugins
    $currUI = 1;
    $pluginUIs = Plugins::addRegFormUI($DB);
    foreach ($pluginUIs as $pluginUI) {
      $this->uis[++$currUI] = $pluginUI;
      $uiClass = get_class($pluginUI);
      $this->uis[$firstUI]->addValidationRules($uiClass::getValidationRules());
    }
    unset($pluginUIs);
    
    $this->title = $this->uis[$firstUI]->getTitle();
    $this->header = $this->uis[$firstUI]->getHeader();
    if(isset($_SESSION['user']) && isset($_SESSION['event']) && $_SESSION['event']->allow_multiple_reg) {
      $this->header .= '  <link rel="stylesheet" href="'.KDbConfig::getCwfHttpPath().'k_db/css/view.css" type="text/css">';
      $this->header .= '  <script src="'.KDbConfig::getCwfHttpPath().'k_db/js/kdb_view.js" type="text/javascript"></script>'."\n";
      $this->header .= "  <script type=\"text/javascript\">
      var uiClassName = 'ViewDataUI';
          
      $(document).ready(function(){
        setupTableEvents(1);
        setupTableFixedHeader();
      });\n";
      $this->header .= "  </script>\n";
    }
    
    if(isset($_POST['kdb_submit']) && $_POST['kdb_submit'] && !isset($_SESSION['registration']['user_id']) &&
       (!isset($_SESSION['registration']['booking_id']) || !isset($_SESSION['event']))) {
      $success = true;
      foreach($this->uis as $ui) {
        if(!$ui->data->fromArray($_POST)) $success = false;
      }
      if($success) {
        foreach($this->uis as $uiIdx => $ui) {
          switch ($uiIdx) {
            case 0:
              if(!$ui->saveForm($DB)) break 2;
              $_SESSION['registration']['user_id'] = $ui->data->user_id;
              $_POST['user_id'] = $_SESSION['registration']['user_id'];
              break;
            case 1:
              if(!$ui->saveForm($DB)) break 2;
              $_SESSION['registration']['booking_id'] = $ui->data->booking_id;
              break;
            default:
              if(!$ui->saveForm($DB)) break ;
              break;
          }
        }
      }
    }
  
    // only send confirmation if new registration / booking
    if(isset($_SESSION['registration']['booking_id']) ||
       (isset($_SESSION['registration']['user_id']) && !isset($_SESSION['event']))) {
      $this->generateConfirmationText($DB);
      $this->sendConfimationEmail();
    }
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // security
  static public function isAllowedAction($action) {
    return in_array($action, array("save"));
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // handle data
  public function setData($data) {
    if (is_a($data, get_class($this->data)))
      $this->data = $data;
    else if (is_array($data))
      $this->data->fromArray($data);
  }
  
  public function getData($asArray=true) {
    return array();
  }
  
  public function getIDName() {
    if ($this->includeBooking)
      return $this->uis[1]->data->getIDName();
    else
      return $this->uis[0]->data->getIDName();
  }
  
  public function getId() {
    if ($this->includeBooking)
      return $this->uis[1]->data->id;
    else
      return $this->uis[0]->data->id;
  }
  
  public function getErrorMessage() {
    if($this->error) return $this->error;
    
    $errorMessage = array();
    foreach ($this->uis as $ui)
      if ($ui->getErrorMessage())
        $errorMessage[] = $ui->getErrorMessage();
    
    return implode("; ", $errorMessage);
  }
  
  static public function getValidationRules() {
    $validationRules = new FormValidation(array(), array());
    return $validationRules;
  }
  
  public function isValid($action) {
    $isValid = true;
    foreach ($this->uis as $ui)
      if(!$ui->isValid($action))
        $isValid = false;
    
    return $isValid;
  }
  
  public function getInvalidFields() {
    $messages = array();
    foreach ($this->uis as $ui)
      $messages = array_merge($messages, $ui->getInvalidFields());
    
    return $messages;
  }
  
  protected function getNewValuesXML($newData) {
    $xml = "  <newValues>\n";
    foreach ($newData as $key => $value) {
      if($key=="passwd") {
        $xml .= "    <newValue name=\"$key\"></newValue>\n";
        $xml .= "    <newValue name=\"{$key}_repeat\"></newValue>\n";
      }
      else if(is_array($value)) {
        $xml .= "    <newValue name=\"$key\">\n      <value>".implode("</value>\n      <value>", $value)."</value>\n    </newValue>\n";
      }
      else
        $xml .= "    <newValue name=\"$key\">$value</newValue>\n";
    }
    $xml .= "  </newValues>\n";
    
    return $xml;
  }
  
  public function getPostBackXML(DbConnection $DB, $action) {
    $xml = "";
    switch ($action) {
      case "updateForm":
        $newData = array();
        foreach($this->uis as $ui)
          $newData = array_merge($newData, $ui->getData());
        $xml .= $this->getNewValuesXML($newData);
        break;
      case "saveasnew":
      case "save":
        $newData = array();
        foreach($this->uis as $ui)
          $newData = array_merge($newData, $ui->data->getPostBackData());
        $xml .= $this->getNewValuesXML($newData);
        break;
    }
    return $xml;
  }
    
  public function doAction(DbConnection $DB, $action) {
    $id = "";
    switch ($action) {
      case "save":
        $this->saveForm($DB);
        $id = $this->getId();
        break;
    }
    return $id;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // Database operations
  public function readData($DB, $conditions) {
    foreach($this->uis as &$ui)
      list($errno, $error) = $ui->readData($DB, $conditions);
    
    return array($errno, $error);
  }
  
  static public function readList($DB) {
    return array();
  }
  public function saveForm(DbConnection $DB) {
    $success = 1;
    foreach ($this->uis as &$ui) {
      if($ui->data->fromArray($_POST)) {
        list($errno, $error) = $ui->data->writeData($DB);
        if ($errno) {
          $this->error = $error;
          $success = 0;
        }
      }
      else {
        $this->error = _("Validation failed.");
        $success = 0;
      }
    }
    return $success;
  }

  public function deleteData(DbConnection $DB, $id) {
    return;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // output
  public function getTitle() {
    return $this->title;
  }
  public function getHeader() {
    return $this->header;
  }
  
  public function getHTML(DbConnection $DB) {
    echo "<form id=\"kdb_form_booking\" action=\"\" method=\"post\" enctype=\"multipart/form-data\">\n";
    echo "<h2 class=\"first\">" . _("Register for") . " \"" . ((isset($_SESSION['event'])) ? $_SESSION['event']->event_title : KDbConfig::getOrganisationName()) . "\"</h2>\n\n";
    
    if(isset($_SESSION['registration']['booking_id']) ||
       (isset($_SESSION['registration']['user_id']) && !isset($_SESSION['event']))) {
      if(isset($this->confirmationText['text_global']))
        echo "<p>".$this->confirmationText['text_global']."</p>\n";
      if(isset($this->confirmationText['text']))
        echo "<p>".$this->confirmationText['text']."</p>\n";
?>
<p>
  <input type="submit" id="reg_finish" name="reg_finish" value="<?=_("Finish")?>">
</p>
<?php
    }
    else if(isset($this->uis[1]) && $this->uis[1]->data->booking_id) {
      $printSubmitButtons = 0;
      foreach ($this->uis as $ui) {
        echo $ui->getHTML($DB);
        $printSubmitButtons = $ui->printSubmitButtons();
      }
      
      if($printSubmitButtons) {
?>
<div class="form_controls_bottom">
<p>
  <input type="submit" id="kdb_save2" name="kdb_save" value="<?=_("Save")?>" onClick="kdbSaveEntry('RegistrationForm', 'save', 'kdb_form_booking');  return false;">
</p>
</div>
<?php
      }
    }
    else if(isset($_SESSION['event']) && !in_array($_SESSION['event']->event_status, array(EVENT_OPEN, EVENT_REGISTERED, EVENT_WAITLIST))) {
      echo "<h3>{$_SESSION['event']->getStatus()}</h3>\n";
      if(THIS_FILE=="registration.php") unset($_SESSION['event']);
    }
    else if(!isset($_SESSION['event']) ||
            !$_SESSION['event']->disclaimer_id ||
            (isset($_POST['kdb_accept']) && $_POST['kdb_accept'])) {
      foreach ($this->uis as $ui)
        echo $ui->getHTMLRegistration($DB);
?>
<p>
  <input type="hidden" id="kdb_accept" name="kdb_accept" value="1">
  <input type="submit" id="kdb_submit" name="kdb_submit" value="<?=_("Submit")?>">
<?php
      if(!isset($_SESSION['user'])) echo '  <input type="submit" id="kdb_abort" name="kdb_abort" class="cancel" value="'._("Abort")."\">\n";
?>
</p>
<?php
    }
    else if (isset($_SESSION['event']) && $_SESSION['event']->disclaimer_id) {
      $disclaimer = new Disclaimer();
      $disclaimer->readData($DB, array("disclaimer_id"=>$_SESSION['event']->disclaimer_id));
      
      echo CWFTools::removeSuspiciousHTML(CWFTools::getHTMLDecodedStr($disclaimer->disclaimer))."\n\n";
?>
<p>
  <input type="submit" id="kdb_accept" name="kdb_accept" value="<?=_("Accept disclaimer")?>">
<?php
      if(!isset($_SESSION['user'])) echo '  <input type="submit" id="kdb_abort" name="kdb_abort" class="cancel" value="'._("Abort")."\">\n";
?>
</p>
<?php
    }
    echo "</form>\n";
    
    if(isset($_SESSION['user']) && !isset($_SESSION['registration']['booking_id']) && isset($_SESSION['event']) && $_SESSION['event']->allow_multiple_reg) {
      $viewUI = new ViewDataUI($DB);
      echo "<h2>"._("Previous bookings")."</h2>\n";
      echo $viewUI->getViewTable($DB);
    }
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // confirmation email / html
  public function sendConfimationEmail() {
    if ($this->sendEmailGlobal)
      KDbTools::sendConfirmation($this->getEmail(), $this->confirmationText['email_global'], KDbConfig::getOrganisationName(), KDbConfig::getRootEmail());
    if($this->sendEmailEvent && isset($_SESSION['event']))
      KDbTools::sendConfirmation($this->getEmail(), $this->confirmationText['email'], $_SESSION['event']->event_title, $_SESSION['event']->event_email, $_SESSION['event']->reg_email);
  }
  
  private function generateConfirmationText(DBConnection $DB) {
    if ($this->sendEmailGlobal) {
      $this->confirmationText['text_global'] = nl2br(KDbConfig::getKdbConfirmationText());
      $this->confirmationText['email_global'] = KDbConfig::getKdbConfirmationEmail();
    }
    if($this->sendEmailEvent && isset($_SESSION['event'])) {
      if($this->includeBooking && is_a($this->uis[1], "UserBookingUI") && $this->uis[1]->data->isWaitlisted()) {
        $this->confirmationText['text'] = nl2br(($_SESSION['event']->confirmation_text_wl ? $_SESSION['event']->confirmation_text_wl : $_SESSION['event']->confirmation_text));
        $this->confirmationText['email'] = ($_SESSION['event']->confirmation_email_wl ? $_SESSION['event']->confirmation_email_wl : $_SESSION['event']->confirmation_email);
      }
      else {
        $this->confirmationText['text'] = nl2br($_SESSION['event']->confirmation_text);
        $this->confirmationText['email'] = $_SESSION['event']->confirmation_email;
      }
    }
    
    if(!$this->includeUser && $_SESSION['user'])
      foreach($this->confirmationText as $name => &$text)
        $text = $_SESSION['user']->getConfirmationText($DB, $text, false);
    foreach ($this->uis as $idx => $ui) {
      foreach($this->confirmationText as $name => &$text)
        if(strstr($name, "text")!==false)
          $text = $ui->data->getConfirmationText($DB, $text, true);
        else
          $text = $ui->data->getConfirmationText($DB, $text, ($idx==0 ? true : false));
    }
    
    foreach ($this->confirmationText as &$text) {
      if(strstr($text, "\$FORM_DATA\$")) {
        if(strstr($name, "text")!==false)
          $text = str_replace("\$FORM_DATA\$", nl2br($this->getFormData($DB)), $text);
        else
          $text = str_replace("\$FORM_DATA\$", $this->getFormData($DB), $text);
      }
    }
  }
  
  private function getEmail() {
    return $this->userData->email;
  }
  
  private function getFormData(DbConnection $DB) {
    $formDataText = "";
    foreach ($this->uis as $ui)
      $formDataText .= $ui->data->getFormData($DB);
    
    return $formDataText;
  }
}
?>
