<?php
require_once("icwfData.inc.php");
require_once("FormValidation.inc.php");

class User implements icwfData {
  private static $noExport = array("passwd");
  private static $notInArray = array("validation", "passwd");
  private $validation;
  
  private $user_id;
  private $reg_date;
  private $familyname;
  private $givenname;
  private $street;
  private $zip_code;
  private $city;
  private $country_id;
  private $phone;
  private $fax;
  private $email;
  private $date_of_birth;
  private $passwd;
  private $gender;
  private $salutation_id;
  private $company;
  private $diseases_allergies;
  private $vegetarian;
  private $paramedic;
  private $cv_filename;
  private $publish_on_website;
  private $allow_email;
  private $last_login;

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  private function initialize() {
    foreach ($this as $key => $value)
      $this->$key = null;
  }
  
  public function __construct(FormValidation $validation=null) {
    $this->initialize();
    
    $this->validation = $validation;
  }
  
  public function __clone() {
    $this->validation = null;
    $this->passwd = null;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // security
  public function checkPW($passwd) {
    if(!$this->passwd) {
      if (!$passwd) 
        return true;
      else
        return false;
    }
    else {
      if (preg_match('~^\$([^$]+)\$(.{'.CWFTools::$SALT_LENGTH.'})~', $this->passwd, $match)) {
        $hashAlgorithm = $match[1];
        $salt = $match[2];
        $pwHash = CWFTools::getPasswordHash($passwd, $hashAlgorithm, $salt);
      }
      else {
        $hashAlgorithm = "md5";
        $salt = "";
        $pwHash = CWFTools::getPasswordHash($passwd, "md5", "");
        $pwHash = preg_replace('~^\$([^$]+)\$~', "", $pwHash);

        // Update password from KDb < v3.0 to new hash algorithm
        if(!$this->last_login && $pwHash == $this->passwd) {
          global $DB;
          $this->passwd = CWFTools::getPasswordHash($passwd);
          $pwHash = $this->passwd;
          $this->changePW($DB, 1);
        }
      }
    }
    return ($pwHash == $this->passwd);
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // get values
  public function __get($name) {
    if($name == "id") $name = "user_id";
    if (array_key_exists($name, get_class_vars(get_class())) && !in_array($name, self::$noExport))
      return $this->$name;
      
    return null;
  }
  
  public function getIDName() {
    return "user_id";
  }
  
  public function toArray() {
    $array = array();
    foreach ($this as $key => $value)
      if(!in_array($key, self::$notInArray)) {
        if($key=="date_of_birth")
          $array[$key] = CWFTools::dateISO2Local($value, getenv("LC_ALL"));
        else
          $array[$key] = $value; 
      }
    return $array;
  }
  
  public function getPostBackData() {
    return array($this->getIDName()=>$this->id, "passwd"=>"");
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // set values
  public function setValidation(FormValidation $validation) {
    $this->validation = $validation;
  }
  
  public function fromArray($array) {
    foreach ($this as $key => $value) {
      switch ($key) {
        case "validation":
        case "reg_date":
        case "last_login":
          break;
        
        case "user_id":
          if(isset($array[$key]))
            $this->$key = $array[$key];
          else
            $this->$key = null;
          break;

        case "gender":
          if(isset($array[$key]) && preg_match("~^m|f$~", $array[$key]))
            $this->$key = $array[$key];
          else
            $this->$key = null;
          break;
          
        case "country_id":
        case "salutation_id":
          if(isset($array[$key]) && preg_match("~^[0-9]+$~", $array[$key]))
            $this->$key = $array[$key];
          else
            $this->$key = null;
          break;
            
        case "vegetarian":
        case "paramedic":
        case "publish_on_website":
        case "allow_email":
          $this->$key = ((isset($array[$key]) && $array[$key]) ? 1 : 0);
          break;
          
        case "passwd":
          if(isset($array[$key]) && isset($array['passwd_repeat']) && $array[$key]==$array['passwd_repeat'])
            $this->$key = CWFTools::getPasswordHash($array[$key]);
          else
            $this->$key = null;
          break;
          
        case "date_of_birth":
          if(isset($array[$key]))
            $this->$key = CWFTools::dateLocal2ISO($array[$key], getenv("LC_ALL"));
          else
            $this->$key = null;
          break;
          
        default:
          if(isset($array[$key]))
            $this->$key = $array[$key];
          else
            $this->$key = null;
          break;
      }
    }
    
    return $this->validation->validate($array);
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - read
  public function readData(DbConnection $DB, $conditions) {
    if(is_array($conditions)) {
      $whereConditions = array();
      foreach ($conditions as $key => $value)
        if(array_key_exists($key, get_class_vars(get_class())))
          $whereConditions[] = "$key=$value";
      
      $conditions = implode(" AND ", $whereConditions);
    }

    $result = $DB->query("SELECT * FROM users WHERE deleted=0 AND ({$conditions});");
    if(!$DB->errno() && $DB->num_rows($result)==1) {
      $user = $DB->fetch_array($result);
      foreach ($user as $property => $value) {
        if(property_exists($this, $property)) {
          switch ($property) {
            default:
              $this->$property = $value;
              break;
          }
        }
      }
      
      return array(0, "");
    }
    else if($DB->errno())
      return array($DB->errno(), $DB->error());
    else if ($DB->num_rows($result)>0)
      return array(1, _("Multiple users found."));
    else
      return array(1, _("User not found."));
  }
  
  static public function readList(DbConnection $DB) {
    $result = $DB->query("SELECT user_id id, concat(familyname, ', ', givenname) name FROM users WHERE deleted=0 ORDER BY name;");
    
    $list = array();
    while ($user = $DB->fetch_array($result))
      $list[] = $user;
      
    return $list;
  }
  
  static public function keyValueExists($value, $user_id) {
    global $DB;
    if(KDbConfig::getEmailDuplicates()==EMAIL_DONT_CHECK)
      return 0;
    else {
      $result = $DB->query("SELECT email FROM users WHERE lower(email)='".strtolower($value)."' AND user_id<>'{$user_id}' AND deleted=0");
      if ($DB->num_rows($result)>0)
        return 1;
      else
        return 0;
    }
    
    return 0;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - write

  public function changePW(DbConnection $DB, $force=0) {
    if(!isset($this->user_id) || !$this->user_id ||
       !isset($this->passwd) || !$this->passwd ||
       (!$force && (!isset($_SESSION['user']->user_id) || $this->user_id<>$_SESSION['user']->user_id)))
      return;
      
    $result = $DB->query("UPDATE users SET
        passwd='{$this->passwd}'
      WHERE user_id='{$this->user_id}';");

    return array($DB->errno(), $DB->error());
  }

  public function hasLoggedIn(DbConnection $DB) {
    if(isset($this->user_id) && $this->user_id) {
      $result = $DB->query("UPDATE users SET
          last_login=NOW()
        WHERE user_id='{$this->user_id}';");
    }
  }
  
  public function writeData(DbConnection $DB) {
    if(isset($this->user_id) && $this->user_id) {
      $result = $DB->query("UPDATE users SET
          familyname='{$this->familyname}',
          givenname='{$this->givenname}',
          street='{$this->street}',
          zip_code='{$this->zip_code}',
          city='{$this->city}',
          country_id=".(!$this->country_id ? "null" : "'{$this->country_id}'").",
          phone='{$this->phone}',
          fax='{$this->fax}',
          email='{$this->email}',
          date_of_birth='{$this->date_of_birth}',
          gender='{$this->gender}',
          salutation_id=".(!$this->salutation_id ? "null" : "'{$this->salutation_id}'").",
          company='{$this->company}',
          diseases_allergies='{$this->diseases_allergies}',
          vegetarian='{$this->vegetarian}',
          paramedic='{$this->paramedic}',
          cv_filename='{$this->cv_filename}',
          publish_on_website='{$this->publish_on_website}',
          allow_email='{$this->allow_email}'
        WHERE user_id='{$this->user_id}';");
      
      $logEntry = "User '$this->user_id' changed.";
    }
    else {
      $result = $DB->query("INSERT INTO users(
          reg_date,
          familyname,
          givenname,
          street,
          zip_code,
          city,
          country_id,
          phone,
          fax,
          email,
          date_of_birth,
          gender,
          salutation_id,
          company,
          diseases_allergies,
          vegetarian,
          paramedic,
          cv_filename,
          publish_on_website,
          allow_email,
          deleted
        ) VALUES(
          NOW(),
          '{$this->familyname}',
          '{$this->givenname}',
          '{$this->street}',
          '{$this->zip_code}',
          '{$this->city}',
          ".(!$this->country_id ? "null" : "'{$this->country_id}'").",
          '{$this->phone}',
          '{$this->fax}',
          '{$this->email}',
          '{$this->date_of_birth}',
          '{$this->gender}',
          ".(!$this->salutation_id ? "null" : "'{$this->salutation_id}'").",
          '{$this->company}',
          '{$this->diseases_allergies}',
          '{$this->vegetarian}',
          '{$this->paramedic}',
          '{$this->cv_filename}',
          '{$this->publish_on_website}',
          '{$this->allow_email}',
          0
        );");
      
      $this->user_id = $DB->insert_id();
      $logEntry = "User '$this->user_id' added.";
    }
    if(!$DB->errno()) {
      if(isset($_SESSION['admin'])) KDbLog::writeLog($DB, $logEntry);
      return array(0, "");
    }
    else
      return array($DB->errno(), $DB->error());
  }
  
  static public function mergeUsers(DbConnection $DB, $ids, $withId) {
    if(!is_array($ids) || !preg_match('~^[0-9]+$~', $withId)) return array(1, "");
    foreach($ids as $id) if(!is_numeric($id)) return array(1, "");
    
    if(in_array($withId, $ids)) $ids = array_diff($ids, array($withId));
    
    $DB->query("UPDATE user_booking SET user_id='{$withId}' WHERE user_id IN ('".implode("', '", $ids)."')");
    self::deleteData($DB, "user_id IN ('".implode("', '", $ids)."')");
    
    return array(0, "");
  }

  static public function unDeleteData(DbConnection $DB, $id) {
    if(preg_match("~^[0-9]+$~", $id))
      $condition = "user_id='{$id}'";
    else
      $condition = $id;
      
    $userIds = $DB->result($DB->query("SELECT group_concat(user_id SEPARATOR ', ') 
                                         FROM users 
                                         WHERE $condition;"), 0, 0);

    $result = $DB->query("UPDATE users SET deleted=0 WHERE {$condition};");
    
    if(!$DB->errno() && $DB->affected_rows()>0) {
      KDbLog::writeLog($DB, "User(s) restored ({$condition}).");
      UserBooking::unDeleteData($DB, "user_id IN ({$userIds})");
      return array(0, "");
    }
    else 
      return array($DB->errno(), $DB->error());
  }
  
  static public function deleteData(DbConnection $DB, $id, $reallyDelete=0) {
    if(preg_match("~^[0-9]+$~", $id))
      $condition = "user_id='{$id}'";
    else
      $condition = $id;
      
    $userIds = $DB->result($DB->query("SELECT group_concat(user_id SEPARATOR ', ') 
                                         FROM users 
                                         WHERE $condition;"), 0, 0);
    if($reallyDelete)
      $result = $DB->query("DELETE FROM users WHERE {$condition};");
    else
      $result = $DB->query("UPDATE users SET deleted=1 WHERE {$condition};");
    
    if(!$DB->errno() && $DB->affected_rows()>0) {
      KDbLog::writeLog($DB, "User(s) ".($reallyDelete ? "finally " : "")."deleted ({$condition}).");
      UserBooking::deleteData($DB, "user_id IN ({$userIds})", $reallyDelete);
      return array(0, "");
    }
    else 
      return array($DB->errno(), $DB->error());
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - misc
  static public function createTable(DbConnection $DB) {
    $result = $DB->query("
      CREATE TABLE users (
        user_id int unsigned NOT NULL AUTO_INCREMENT PRIMARY KEY,
        reg_date datetime,
        familyname text,
        givenname text,
        street text,
        zip_code text,
        city text,
        country_id int unsigned,
        phone text,
        fax text,
        email text,
        date_of_birth date,
        passwd text,
        gender char(1),
        salutation_id int unsigned,
        company text,
        diseases_allergies text,
        vegetarian tinyint,
        paramedic tinyint,
        cv_filename text,
        publish_on_website tinyint,
        allow_email tinyint,
        last_login datetime,
        deleted tinyint
      );");
    if ($result) { echo _("&nbsp;&nbsp;- creating table").": 'user'.<br>\n"; }
    else { echo _("&nbsp;&nbsp;<b>- Error when creating table</b>")." 'user'<br>\n"; $error = 1; }
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // output of formatted data
  static public function getProperties($property=null) {
    $properties = array(
      'user_id' => _("User Id"),
      'reg_date' => _("Registration date"),
      'familyname' => _("Family name"),
      'givenname' => _("Given name"),
      'street' => _("Street"),
      'zip_code' => _("ZIP code"),
      'city' => _("City"),
      'country_id' => _("Country"),
      'phone' => _("Phone"),
      'fax' => _("Fax"),
      'email' => _("Email"),
      'date_of_birth' => _("Date of birth"),
      'passwd' => _("Password"),
      'gender' => _("Gender"),
      'salutation_id' => _("Salutation"),
      'company' => _("Company"),
      'diseases_allergies' => _("Diseases / allergies"),
      'vegetarian' => _("Vegetarian"),
      'paramedic' => _("Paramedic"),
      'cv_filename' => _("Curriculum vitae"),
      'publish_on_website' => _("Publish on website"),
      'allow_email' => _("Allow email"),
      'last_login' => _("Last login")
    );
    
    if(is_array($property)) {
      $newProperties = array();
      foreach($property as $id=>$p)
        if(isset($properties[$p])) $newProperties[$id] = $properties[$p];
      return $newProperties;
    }
    else if($property) {
      $property = preg_replace('~^([^.]+\.)?([^.]+)$~', '\2', $property);
      return ((isset($properties[$property])) ? $properties[$property] : null);
    }
    else
      return $properties;
  }

  static public function getPropertySelect($columnNames, $addTableName="") {
    if(is_array($columnNames)) {
      foreach($columnNames as &$columnName)
        $columnName = self::getPropertySelect($columnName, $addTableName);
    }
    else {
      $property = preg_replace('~^([^.]+\.)?([^.]+)$~', '\2', $columnNames);
      if(!array_key_exists($property, get_class_vars(get_class()))) return $columnNames;

      $columnNames = $addTableName . $property;
      switch ($property) {
        case "salutation_id":
          $columnNames = "(SELECT salutation FROM salutations WHERE salutation_id={$columnNames}) $property";
          break;
        case "country_id":
        case "country":
          $columnNames = "(SELECT country FROM countries WHERE country_id={$columnNames}) $property";
          break;
        case "vegetarian":
        case "paramedic":
        case "publish_on_website":
        case "allow_email":
          $columnNames = "IF({$columnNames}>0, '"._("yes")."', '"._("no")."') {$property}";
          break;
      }
    }
    return $columnNames; 
  }
  
  public function getText($DB, $field) {
    $yesNo = array(_("no"), _("yes"));
    
    switch ($field) {
      case "salutation_id":
      case "salutation":
        return $DB->result($DB->query("SELECT salutation FROM salutations WHERE salutation_id={$this->salutation_id};"), 0, 0);
        break;
      case "country_id":
      case "country":
        return $DB->result($DB->query("SELECT country FROM countries WHERE country_id={$this->country_id};"), 0, 0);
        break;
      case "vegetarian":
      case "paramedic":
      case "publish_on_website":
      case "allow_email":
        return $yesNo[$this->$field];
        break;
      case "cv_filename":
        return $this->$field;
        break;
      case "date_of_birth":
        return CWFTools::dateISO2Local($this->$field, getenv("LC_ALL"));
        break;
      
      default:
        if(array_key_exists($field, get_class_vars(get_class()))) {
          return $this->$field;
        }
        break;
    }
    return null;    
  }
  
  public function getFormData($DB) {
    $formDataText = "";
    foreach ($this as $key => $value)
      if (!in_array($key, self::$notInArray) && $value) {
        $formDataText .= self::getProperties($key).": ".$this->getText($DB, $key)."\n";
      }
    
    return $formDataText;
  }
  
  public function getConfirmationText(DbConnection $DB, $confirmationText, $generatePW=false) {
    $pw = "";
    if ($generatePW) {
      if (KDbConfig::getAllowUserLogin()) {
        if (!isset($_SESSION['reg_passwd'])) {
          $pw = CWFTools::generatePassword();
        }
        else 
          $pw = $_SESSION['reg_passwd'];
        $this->passwd = CWFTools::getPasswordHash($pw);
        
        if (!isset($_SESSION['reg_passwd']))
          $this->changePW($DB, 1);
  
        $_SESSION['reg_passwd'] = $pw;
      }
    }
    
    foreach ($this as $key => $value)
      if(strstr($confirmationText, "\${$key}\$")) {
        if($key=="passwd") {
          if($pw)
            $confirmationText = str_replace("\${$key}\$", $pw, $confirmationText);
          else
            $confirmationText = str_replace("\${$key}\$", _("not set/changed"), $confirmationText);
        }
        else if (!in_array($key, self::$notInArray)) {
          $confirmationText = str_replace("\${$key}\$", $this->getText($DB, $key), $confirmationText);
        }
      }
      if(strstr($confirmationText, "\$salutation\$"))
        $confirmationText = str_replace("\$salutation\$", $this->getText($DB, "salutation"), $confirmationText);
    return $confirmationText;
  }
}
?>
