<?php
require_once("icwfData.inc.php");
require_once("FormValidation.inc.php");

class PCGSelectChar implements icwfData {
  private static $noExport = array("");
  private static $notInArray = array("validation", "");
  private $validation;
  
  private $booking_id;
  private $charakter_id;
  private $end_char_view;

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  private function initialize() {
    foreach ($this as $key => $value)
      $this->$key = null;
    
    if(preg_match("~([0-9]{1,2}\.\s*([0-9]{1,2}\.|[A-Za-z]+)\s*[0-9]{2,4})\s*$~", $_SESSION['event']->event_period, $match)) {
      $months = array("Jan(uar)?", "Feb(ruar)?", "Mär(z)?", "Apr(il)?", "Mai", "Jun(i)?", "Jul([iy])?", "Aug(ust)?", "Sep(tember)?", "Okt(ober)?", "Nov(ember)?", "Dez(ember)?");
      $date = preg_replace("~([0-9]{1,2}\.)\s*([0-9]{1,2}\.|[A-Za-z]+)\s*([0-9]{2,4})\s*$~", "\\1\\2\\3", $match[1]);
      for($i=0; $i<count($months); $i++)
        $date = preg_replace("~{$months[$i]}~", sprintf("%02d.", $i+1), $date);
      $this->end_char_view = CWFTools::dateLocal2ISO($date, "de_DE");
    }
    else
      $this->end_char_view=null;
  }
  
  public function __construct(FormValidation $validation=null) {
    $this->initialize();
    
    $this->validation = $validation;
  }
  
  public function __clone() {
    $this->validation = null;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // security

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // get values
  public function __get($name) {
    if($name == "id") $name = "booking_id";
    if (array_key_exists($name, get_class_vars(get_class())) && !in_array($name, self::$noExport))
      return $this->$name;
      
    return null;
  }
  
  public function getIDName() {
    return "booking_id";
  }
  
  public function toArray() {
    $array = array();
    foreach ($this as $key => $value)
      if(!in_array($key, self::$notInArray)) {
        $array[$key] = $value; 
      }
    return $array;
  }
  
  public function getPostBackData() {
    return array($this->getIDName()=>$this->id);
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // set values
  public function setValidation(FormValidation $validation) {
    $this->validation = $validation;
  }
  
  public function fromArray($array) {
    foreach ($this as $key => $value) {
      switch ($key) {
        case "validation":
          break;
        
        case "booking_id":
          if(isset($_SESSION['registration']['booking_id']) && $_SESSION['registration']['booking_id'])
            $this->booking_id = $_SESSION['registration']['booking_id'];
          else if(isset($array[$key]) && preg_match("~^[0-9]+$~", $array[$key]))
            $this->$key = $array[$key];
          else
            $this->$key = null;
          break;
          
        case "charakter_id":
          if(isset($array[$key]) && preg_match("~^[0-9]+$~", $array[$key]))
            $this->$key = $array[$key];
          else
            $this->$key = null;
          break;
            
        case "end_char_view":
          if(isset($array[$key]))
            $this->$key = CWFTools::dateLocal2ISO($array[$key], getenv("LC_ALL"));
          else
            $this->$key = null;
          break;
          
        default:
          if(isset($array[$key]))
            $this->$key = $array[$key];
          else
            $this->$key = null;
          break;
      }
    }
    return $this->validation->validate($array);
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - read
  public function readData(DbConnection $DB, $conditions) {
    if(is_array($conditions)) {
      $whereConditions = array();
      foreach ($conditions as $key => $value)
        if(array_key_exists($key, get_class_vars(get_class())))
          $whereConditions[] = "$key=$value";
      
      $conditions = implode(" AND ", $whereConditions);
    }
    
    $result = $DB->query("SELECT * FROM pcg_reg WHERE {$conditions};");
    if(!$DB->errno() && $DB->num_rows($result)==1) {
      $user = $DB->fetch_array($result);
      foreach ($user as $property => $value) {
        if(property_exists($this, $property)) {
          switch ($property) {
            case "end_char_view":
              if($value!="0000-00-00" && $value!=null)
                $this->$property = $value;
              break;
            default:
              $this->$property = $value;
              break;
          }
        }
      }
      
      return array(0, "");
    }
    else if($DB->errno())
      return array($DB->errno(), $DB->error());
    else if ($DB->num_rows($result)>0)
      return array(1, _("Multiple entries found."));
    else
      return array(1, _("No Entries found."));
  }
  
  static public function readList(DbConnection $DB) {
    $list = array();
    return $list;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - write
  public function writeData(DbConnection $DB) {
    if(!isset($this->booking_id) || !$this->booking_id) return array(1, _("ID not set."));
    
    if(!$this->end_char_view && !$this->charakter_id) return self::deleteData($DB, $this->booking_id);
    
    $result = $DB->query("UPDATE pcg_reg SET
        charakter_id='{$this->charakter_id}',
        end_char_view=".(!$this->end_char_view ? "null" : "'{$this->end_char_view}'")."
      WHERE booking_id='{$this->booking_id}';");
    
    if ($DB->affected_rows()<=0) {
      $result = $DB->query("INSERT INTO pcg_reg(
          booking_id,
          charakter_id,
          end_char_view
        ) VALUES(
          '{$this->booking_id}',
          '{$this->charakter_id}',
          ".(!$this->end_char_view ? "null" : "'{$this->end_char_view}'")."
        );");
    }
    return array($DB->errno(), $DB->error());
  }

  static public function deleteData(DbConnection $DB, $id) {
    if(!preg_match("~^[0-9]+$~", $id))
      return array(1, "ID not valid.");
      
    $result = $DB->query("DELETE FROM pcg_reg WHERE booking_id='{$id}';");

    if(!$DB->errno() && $DB->affected_rows()>0)
      return array(0, "");
    else 
      return array($DB->errno(), $DB->error());
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - misc
  static public function createTable(DbConnection $DB) {
    $result = $DB->query("
      CREATE TABLE pcg_reg (
        booking_id int unsigned NOT NULL PRIMARY KEY,
        charakter_id int unsigned NOT NULL,
        end_char_view date
      );");
    if ($result) { echo _("&nbsp;&nbsp;- creating table").": 'pcg_reg'.<br>\n"; }
    else { echo _("&nbsp;&nbsp;<b>- Error when creating table</b>")." 'pcg_reg'<br>\n"; $error = 1; }
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // output of formatted data
  static public function getProperties($property=null) {
    $properties = array(
      'charakter_id' => "Charakter Name",
      'charakter_link' => "Charakter",
      'is_neu' => "Neuer Charakter",
      'fertigkeiten' => "Fertigkeiten",
      'rasse' => "Rasse",
      'gottheit' => "Gottheit",
      'herkunft' => "Herkunft",
      'groesse'=>"Größe",
      'gewicht'=>"Gewicht",
      'haarfarbe'=>"Haarfarbe",
      'augenfarbe'=>"Augenfarbe",
      'beruf'=>"Beruf",
      'kleidung'=>"Kleidung",
      'bes_kennzeichen'=>"Bes. Kennzeichen",
      'is_dieb'=>"Dieb",
      'is_faelscher'=>"Fälscher",
      'is_meuchler'=>"Meuchler",
      'is_ortskundig'=>"Ortskundig",
      'is_legendenkenntnis'=>"Legendenkenntnis",
      'is_reich'=>"Reichtum",
      'is_zauberkundig'=>"Zauberkundig",
      'charakter_bild'=>"Charakter Bild",
      'diebeszinken'=>"Diebeszinken"
    );
    
    if(is_array($property)) {
      $newProperties = array();
      foreach($property as $id=>$p)
        if(isset($properties[$p])) $newProperties[$id] = $properties[$p];
      return $newProperties;
    }
    else if($property) {
      $property = preg_replace('~^([^.]+\.)?([^.]+)$~', '\2', $property);
      return ((isset($properties[$property])) ? $properties[$property] : null);
    }
    else
      return $properties;
  }

  static public function getPropertySelect($columnNames, $addTableName="") {
    if(is_array($columnNames)) {
      foreach($columnNames as &$columnName)
        $columnName = self::getPropertySelect($columnName, $addTableName);
    }
    else {
      $property = preg_replace('~^([^.]+\.)?([^.]+)$~', '\2', $columnNames);
      $columnNames = "";
      //if(!array_key_exists($property, get_class_vars(get_class()))) return $columnNames;

      //$columnNames = $addTableName . $property;
      if(isset($_SESSION['event'])) {
        switch ($property) {
          case "charakter_id":
            $columnNames = "(SELECT charakter_name ". 
                "FROM pcg_charaktere pc ".
                  "JOIN pcg_reg pcr ON pcr.charakter_id=pc.charakter_id ".
                "WHERE booking_id=ub.booking_id) $property";
            break;
          case "is_neu":
            $columnNames = "(SELECT if(pc.spiele='array (\n)', 'ja', 'nein') ".
                "FROM pcg_charaktere pc ".
                  "JOIN pcg_reg pcr ON pcr.charakter_id=pc.charakter_id ".
                "WHERE booking_id=ub.booking_id) $property";
            break;
          case "groesse":
          case "gewicht":
          case "haarfarbe":
          case "augenfarbe":
          case "beruf":
          case "kleidung":
          case "bes_kennzeichen":
            $columnNames = "(SELECT $property ".
                "FROM pcg_charaktere pc ".
                  "JOIN pcg_reg pcr ON pcr.charakter_id=pc.charakter_id ".
                "WHERE booking_id=ub.booking_id) $property";
            break;
          case "is_dieb":
            $filter = "pf.fertigkeit='Stehlen'";
          case "is_faelscher":
            if(!isset($filter)) $filter = "pf.fertigkeit='Fälschen'";
          case "is_meuchler":
            if(!isset($filter)) $filter = "pf.fertigkeit='Meucheln'";
          case "is_ortskundig":
            if(!isset($filter)) $filter = "pf.fertigkeit='Ortskenntnis'";
          case "is_legendenkenntnis":
            if(!isset($filter)) $filter = "pf.fertigkeit='Legendenkenntnis'";
          case "is_reich":
            if(!isset($filter)) $filter = "pf.fertigkeit='Reichtum'";
          case "is_zauberkundig":
            if(!isset($filter)) $filter = "pf.fertigkeit IN ('Zauberei', 'Klerikalmagie', 'Schamanismus', 'Bardenmagie') ";
            $columnNames = "(SELECT (SELECT if(count(fertigkeit)>0, 'ja', 'nein') ".
                        "FROM pcg_char_fertigkeiten pcf ".
                          "JOIN pcg2_fertigkeiten pf ON pf.fertigkeit_id=pcf.fertigkeit_id ".
                        "WHERE pcf.charakter_id=pc.charakter_id ".
                          "AND {$filter}) ". 
                  "FROM pcg_charaktere pc ".
                    "JOIN pcg_reg pcr ON pcr.charakter_id=pc.charakter_id ".
                  "WHERE pcr.booking_id=ub.booking_id) $property";
            break;
          case "charakter_bild":
            $columnNames = "(SELECT concat(u.familyname, '_', pc.charakter_name, '_bild', '::', pc.charakter_bild) ".
                "FROM pcg_charaktere pc ".
                  "JOIN pcg_reg pcr ON pcr.charakter_id=pc.charakter_id ".
                "WHERE booking_id=ub.booking_id) $property";
            break;
          case "diebeszinken":
            $columnNames = "(SELECT concat(u.familyname, '_', pc.charakter_name, '_zinken', '::', pc.diebeszinken) ".
                "FROM pcg_charaktere pc ".
                  "JOIN pcg_reg pcr ON pcr.charakter_id=pc.charakter_id ".
                "WHERE booking_id=ub.booking_id) $property";
            break;
          case "charakter_link":
            $columnNames = "(SELECT concat('<a href=\"../PluginWrapper.php/pcg2/CharakterAnzeigen?charakter_id=', pc.charakter_id, '\" target=\"_blank\">', pc.charakter_name, '</a>') ". 
                "FROM pcg_charaktere pc ".
                  "JOIN pcg_reg pcr ON pcr.charakter_id=pc.charakter_id ".
                "WHERE pcr.booking_id=ub.booking_id ".
                  "AND pcr.end_char_view>=NOW()) $property";
            break;
          case "fertigkeiten":
            if(!isset($_SESSION['filter']['fertigkeiten']))
              $columnNames = "'' $property";
            else {
              if(preg_match('~^/(.*)/$~', $_SESSION['filter']['fertigkeiten'], $match))
                $filter = "pf.fertigkeit REGEXP '{$match[1]}'";
              else
                $filter = "pf.fertigkeit LIKE '".str_replace(array("%", "_", "*", "?"), array("\\%", "\\_", "%", "_"), $_SESSION['filter']['fertigkeiten'])."'";
              $columnNames = "(SELECT group_concat(fertigkeit ORDER BY fertigkeit SEPARATOR ', ') ". 
                  "FROM pcg_charaktere pc ".
                    "JOIN pcg_reg pcr ON pcr.charakter_id=pc.charakter_id ".
                    "JOIN pcg_char_fertigkeiten pcf ON pcf.charakter_id=pc.charakter_id ".
                    "JOIN pcg2_fertigkeiten pf ON pf.fertigkeit_id=pcf.fertigkeit_id ".
                  "WHERE pcr.booking_id=ub.booking_id ".
                    "AND {$filter}) $property";
            }
            break;
          case "rasse":
            $columnNames = "(SELECT pr.rasse ". 
                "FROM pcg_charaktere pc ".
                  "JOIN pcg_reg pcr ON pcr.charakter_id=pc.charakter_id ".
                  "JOIN pcg2_rassen pr ON pr.rasse_id=pc.rasse_id ".
                "WHERE booking_id=ub.booking_id) $property";
            break;
          case "gottheit":
            $columnNames = "(SELECT pg.gottheit ". 
                "FROM pcg_charaktere pc ".
                  "JOIN pcg_reg pcr ON pcr.charakter_id=pc.charakter_id ".
                  "JOIN pcg2_goetter pg ON pg.gottheit_id=pc.gottheit_id ".
                "WHERE booking_id=ub.booking_id) $property";
            break;
          case "herkunft":
            $columnNames = "(SELECT concat(pl.land_name, ' / ', pc.herkunft) ". 
                "FROM pcg_charaktere pc ".
                  "JOIN pcg_reg pcr ON pcr.charakter_id=pc.charakter_id ".
                  "JOIN pcg_laender pl ON pl.land_id=pc.land_id ".
                "WHERE booking_id=ub.booking_id) $property";
            break;
        }
      }
      else if($property=="charakter_id") {
        $columnNames = "(SELECT group_concat(charakter_name ORDER BY charakter_name SEPARATOR '<br>') ". 
                "FROM pcg_charaktere pc ".
                "WHERE pc.user_id=u.user_id) $property";
      }
    }
    return $columnNames; 
  }
  
  public static function getTextView($field, $value) {
    switch ($field) {
      case "charakter_bild":
      case "diebeszinken":
        $newValue = preg_replace('~^(.*)::(.*)\.([^.]+)$~', "$1.$3", $value);
        return preg_replace('/[^.a-zA-Z0-9ÄÖÜäöüß_-]/u', '', $newValue);
        break;
      default:
        return $value;                  
    }
  }
  
  private function getText($DB, $field) {
    $yesNo = array(_("no"), _("yes"));
    
    return null;    
  }
  
  public function getFormData($DB) {
    $formDataText = "";
    return $formDataText;
  }
  
  public function getConfirmationText(DbConnection $DB, $confirmationText) {
    return $confirmationText;
  }
}
?>
